package com.jz.dmp.modules.controller.DataIntegration;

import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.BasePageBean;
import com.jz.common.page.PageInfoResponse;
import com.jz.dmp.modules.controller.DataIntegration.bean.*;
import com.jz.dmp.modules.service.OfflineSynchService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:离线同步
 * @ClassName: OfflineSynchController
 * @Author Bellamy
 * @Date 2020/12/20
 */
@RestController
@RequestMapping("/offlineSynch")
@Api(tags = "数据集成--离线同步")
public class OfflineSynchController {

    @Autowired
    private OfflineSynchService offlineSynchService;

    /**
     * 离线同步任务列表分页查询
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "任务列表分页查询", notes = "任务列表分页查询")
    @PostMapping(value = "/taskListPage")
    public PageInfoResponse<TaskListPageDto> getTaskListPage(@RequestBody @Validated TaskListPageReq taskListPageReq) {
        PageInfoResponse<TaskListPageDto> pageInfo = new PageInfoResponse<TaskListPageDto>();
        try {
            pageInfo = offlineSynchService.queryTaskListPage(taskListPageReq);
        } catch (Exception e) {
            pageInfo.setMessage("查询失败");
            pageInfo.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return pageInfo;
    }

    /**
     * 获取源数据库名称——下拉框
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "获取源数据库名称-下拉框", notes = "获取源数据库名称")
    @GetMapping(value = "/sourceDbList")
    @ApiImplicitParam(name = "projectId", value = "项目id",required = true)
    public JsonResult<List<SourceDbNameListDto>> getSourceDbList(@RequestParam Integer projectId) throws Exception {
        JsonResult<List<SourceDbNameListDto>> jsonResult = offlineSynchService.querygSourceDbList(projectId);
        return jsonResult;
    }

    /**
     * 根据源数据库id,获取源数据表——下拉框
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "根据源数据库id,获取源数据表-下拉框", notes = "根据源数据库id,获取源数据表")
    @GetMapping(value = "/sourceTableList")
    @ApiImplicitParam(name = "sourceDbId", value = "源数据库id")
    public JsonResult getSourceTableList(@RequestParam Long sourceDbId, @RequestParam(value = "targetName", required = false) String targetName) throws Exception {
        JsonResult list = offlineSynchService.querygSourceTableList(sourceDbId, targetName);
        return list;
    }

    /**
     * 任务立即运行
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "任务立即运行", notes = "任务立即运行")
    @GetMapping(value = "/taskRunNowByTaskId")
    @ApiImplicitParam(name = "taskId", value = "任务id")
    public JsonResult getTaskRunNowByTaskId(@RequestParam(value = "taskId") String taskId) throws Exception {
        if (StringUtils.isEmpty(taskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR);
        }
        JsonResult list = offlineSynchService.taskRunNowByTaskId(taskId);
        return list;
    }

    /**
     * 根据taskId删除离线任务
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "删除任务", notes = "删除任务")
    @GetMapping(value = "/delTaskByTaskId")
    @ApiImplicitParam(name = "taskId", value = "任务id")
    public JsonResult delTaskByTaskId(@RequestParam(value = "taskId") String taskId) throws Exception {
        if (StringUtils.isEmpty(taskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR);
        }
        JsonResult list = offlineSynchService.delTaskByTaskId(taskId);
        return list;
    }

    /**
     * 状态查看列表分页查询
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "状态查看", notes = "状态查看列表分页查询")
    @PostMapping(value = "/checkTaskStatus")
    public JsonResult<List<CheckTaskStatusPageDto>> getCheckTaskStatusListPage(@RequestBody @Validated CheckTaskStatusPageReq checkTaskStatusPageReq) throws Exception {
        JsonResult<List<CheckTaskStatusPageDto>> list = offlineSynchService.queryCheckTaskStatusListPage(checkTaskStatusPageReq);
        return list;
    }

    /**
     * 校验状态详情--查看 规则执行结果
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "校验状态详情--查看", notes = "校验状态详情--查看")
    @PostMapping(value = "/checkJyStatusInfo")
    public PageInfoResponse<CheckJyRlueStatusDto> getCheckStatusInfo(@RequestBody @Validated CheckJyRlueStatusReq checkJyRlueStatusReq) throws Exception {
        PageInfoResponse<CheckJyRlueStatusDto> pageInfo = new PageInfoResponse<CheckJyRlueStatusDto>();
        try {
            pageInfo = offlineSynchService.selectCheckJyStatusInfo(checkJyRlueStatusReq);
        } catch (Exception e) {
            pageInfo.setMessage("查询失败");
            pageInfo.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return pageInfo;
    }

    /**
     * 获取源表和目标表的字段
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "获取源表和目标表的字段", notes = "获取源表和目标表的字段")
    @PostMapping(value = "/getSoureAndTargetColumns")
    public JsonResult getSoureAndTargetColumns(@RequestBody @Validated SoureAndTargetColumnsReq soureAndTargetColumnsReq) throws Exception {
        JsonResult list = offlineSynchService.querySoureAndTargetColumnsByParams(soureAndTargetColumnsReq);
        return list;
    }

    /**
     * 校验规则
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "校验规则", notes = "校验规则")
    @PostMapping(value = "/findListJyRule")
    public PageInfoResponse<DvRuleTDto> findListJyRule(@RequestBody BasePageBean basePageBean, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DvRuleTDto> pageInfo = new PageInfoResponse<DvRuleTDto>();
        try {
            pageInfo = offlineSynchService.queryJyRuleListPage(basePageBean, httpRequest);
        } catch (Exception e) {
            pageInfo.setMessage("查询失败");
            pageInfo.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return pageInfo;
    }

    /**
     * 保存离线任务数据
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "保存离线任务数据", notes = "保存离线任务数据")
    @PostMapping(value = "/addSyncTask")
    public JsonResult addSyncTask(@RequestBody @Validated SyncDmpTaskAddReq syncDmpTaskAddReq) throws Exception {
        JsonResult list = offlineSynchService.addSyncTask(syncDmpTaskAddReq);
        return list;
    }

    /**
     * 编辑离线任务数据
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "编辑离线任务数据", notes = "编辑离线任务数据")
    @PostMapping(value = "/updateSyncTask")
    public JsonResult updateSyncTask(@RequestBody @Validated SyncDmpTaskAddReq syncDmpTaskAddReq) throws Exception {
        JsonResult list = offlineSynchService.updateSyncTask(syncDmpTaskAddReq);
        return list;
    }
}
