package com.jz.dmp.modules.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.jz.common.constant.JsonResult;
import com.jz.common.utils.web.HttpClientUtils;
import com.jz.dmp.modules.controller.dataService.bean.OrganizationManageAddReq;
import com.jz.dmp.modules.controller.dataService.bean.OrganizationManageDetailQueryReq;
import com.jz.dmp.modules.controller.dataService.bean.OrganizationManageListQueryReq;
import com.jz.dmp.modules.controller.dataService.bean.OrganizationManageUpdateReq;
import com.jz.dmp.modules.service.DmpOrgMangeService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @ClassName: DmpOrgMangeServiceImpl
 * @Description: 数据服务组织管理
 * @Author: Bellamy
 * @Date 2021/1/15
 * @Version 1.0
 */
@Service("dmpOrgMangeService")
public class DmpOrgMangeServiceImpl implements DmpOrgMangeService {

    private static Logger logger = LoggerFactory.getLogger(DmpOrgMangeServiceImpl.class);

    //列表分页查询url
    private static final String orgListPage = "/api/organization/listOrg";

    //删除组织url
    private static final String delOrg = "/api/organization/logoutOrg";

    //新增组织url
    private static final String addOrg = "/api/organization/add";

    //编辑组织url
    private static final String updateOrg = "/api/organization/update";

    //根据组织id获取组织详情url
    private static final String orgDetail = "/api/organization/getOrgDetail";

    @Value("${spring.gateway-url}")
    private String gatewayUrl;

    /**
     * 列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-16
     */
    @Override
    public JsonResult queryOrgListPage(OrganizationManageListQueryReq req) throws Exception {
        JsonResult result = new JsonResult();
        String url = gatewayUrl + orgListPage;

        if (StringUtils.isNotEmpty(req.getOrgName())) {
            req.setOrgName(req.getOrgName().trim());
        }
        String resultData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
        if (StringUtils.isEmpty(resultData)) {
            throw new RuntimeException("查询失败！");
        }
        logger.info("#################组织管理列数据{}" + resultData);
        Map jsonObject = JSONObject.parseObject(resultData);
        if (jsonObject != null) {
            result.setData(jsonObject.get("data"));
            /*Map records = JSONObject.parseObject(jsonObject.get("data").toString());
            List<Map> list = (List<Map>) records.get("records");*/
        }
        return result;
    }

    /**
     * 删除组织
     *
     * @return
     * @author Bellamy
     * @since 2021-01-16
     */
    @Override
    public JsonResult delOrgById(long id) throws Exception {
        Map params = new HashMap();
        params.put("id", id);
        String url = gatewayUrl + delOrg;
        String returnData = HttpClientUtils.getJsonForParam(url, params);
        if (StringUtils.isEmpty(returnData)) {
            throw new RuntimeException("删除失败！");
        }
        logger.info("#################响应结果{}" + returnData);
        Map map = JSONObject.parseObject(returnData);
        if ("200".equals(map.get("code").toString())) {
            return JsonResult.ok();
        }
        return JsonResult.error("删除失败！");
    }

    /**
     * 新增组织
     *
     * @return
     * @author Bellamy
     * @since 2021-01-16
     */
    @Override
    public JsonResult addOrg(OrganizationManageAddReq req) throws Exception {
        String url = gatewayUrl + addOrg;
        String returnData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
        if (StringUtils.isEmpty(returnData)) {
            throw new RuntimeException("新增失败！");
        }
        logger.info("#################响应结果{}" + returnData);
        Map map = JSONObject.parseObject(returnData);
        if ("200".equals(map.get("code").toString())) {
            return JsonResult.ok();
        }
        logger.info(map.get("message").toString());
        return JsonResult.error(map.get("message").toString());
    }

    /**
     * 编辑组织
     *
     * @return
     * @author Bellamy
     * @since 2021-01-16
     */
    @Override
    public JsonResult updateOrg(OrganizationManageUpdateReq req) throws Exception {
        String url = gatewayUrl + updateOrg;
        String returnData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
        if (StringUtils.isEmpty(returnData)) {
            throw new RuntimeException("编辑失败！");
        }
        logger.info("#################响应结果{}" + returnData);
        Map map = JSONObject.parseObject(returnData);
        if ("200".equals(map.get("code").toString())) {
            return JsonResult.ok();
        }
        logger.info(map.get("message").toString());
        return JsonResult.error(map.get("message").toString());
    }

    /**
     * 根据组织id获取组织详情
     *
     * @author Bellamy
     * @since 2021-01-18
     */
    @Override
    public JsonResult getOrgInfoByOrgId(String id) throws Exception {
        String url = gatewayUrl + orgDetail;
        OrganizationManageDetailQueryReq req = new OrganizationManageDetailQueryReq();
        req.setId(Long.valueOf(id));
        String returnData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
        if (StringUtils.isEmpty(returnData)) {
            throw new RuntimeException("查询失败！");
        }
        logger.info("#################响应结果{}" + returnData);
        Map map = JSONObject.parseObject(returnData);
        if ("200".equals(map.get("code").toString())) {
            return JsonResult.ok(map.get("data"));
        }
        logger.info(map.get("message").toString());
        return JsonResult.error(map.get("message").toString());
    }
}
