package com.jz.dmp.modules.service.impl;


import com.amazonaws.services.dynamodbv2.xspec.M;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.realTime.DBUtil;
import com.jz.common.utils.realTime.RestClient;
import com.jz.common.utils.web.SessionUtils;
import com.jz.dmp.modules.controller.DataIntegration.bean.*;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.dao.DmpRealtimeSyncHandleCountDao;
import com.jz.dmp.modules.dao.DmpRealtimeSyncInfoDao;
import com.jz.dmp.modules.model.*;
import com.jz.dmp.modules.service.DmpRealtimeSyncInfoService;
import freemarker.core.ParseException;
import freemarker.template.MalformedTemplateNameException;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateNotFoundException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfig;

import java.io.IOException;
import java.io.StringWriter;
import java.util.*;
import java.util.regex.Pattern;

/**
 * 实时同步任务(DmpRealtimeSyncInfo)表服务实现类
 *
 * @author Bellamy
 * @since 2021-01-05 14:18:03
 */
@Service("dmpRealtimeSyncInfoService")
@Transactional
public class DmpRealtimeSyncInfoServiceImpl implements DmpRealtimeSyncInfoService {

    private static Logger logger = LoggerFactory.getLogger(DmpRealtimeSyncInfoServiceImpl.class);

    @Value("${spring.public-key}")
    private String publicKey;

    @Autowired
    private FreeMarkerConfig freeMarkerConfig;

    @Autowired
    private DmpRealtimeSyncInfoDao dmpRealtimeSyncInfoDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpRealtimeSyncHandleCountDao dmpRealtimeSyncHandleCountDao;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo queryById(Integer id) {
        return this.dmpRealtimeSyncInfoDao.queryById(id);
    }

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    @Override
    public List<DmpRealtimeSyncInfo> queryAllByLimit(int offset, int limit) {
        return this.dmpRealtimeSyncInfoDao.queryAllByLimit(offset, limit);
    }

    /**
     * 新增数据
     *
     * @param dmpRealtimeSyncInfo 实例对象
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo insert(DmpRealtimeSyncInfo dmpRealtimeSyncInfo) {
        this.dmpRealtimeSyncInfoDao.insert(dmpRealtimeSyncInfo);
        return dmpRealtimeSyncInfo;
    }

    /**
     * 修改数据
     *
     * @param dmpRealtimeSyncInfo 实例对象
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo update(DmpRealtimeSyncInfo dmpRealtimeSyncInfo) {
        this.dmpRealtimeSyncInfoDao.update(dmpRealtimeSyncInfo);
        return this.queryById(dmpRealtimeSyncInfo.getId());
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public boolean deleteById(Integer id) {
        return this.dmpRealtimeSyncInfoDao.deleteById(id) > 0;
    }

    /**
     * 实时同步任务列表分页查询
     *
     * @return
     * @author Bellamy
     */
    @Override
    public PageInfoResponse<RealTimeSyncListDto> queryRealTimeSyncListPage(RealTimeSyncListReq req) throws Exception {
        PageInfoResponse<RealTimeSyncListDto> pageInfoResponse = new PageInfoResponse<>();

        if (StringUtils.isNotEmpty(req.getSourceDatabaseName())) { //来源数据源名称
            req.setSourceDatabaseName(req.getSourceDatabaseName().trim());
        }
        if (StringUtils.isNotEmpty(req.getTargetDatabaseName())) { //目标源数据源名称
            req.setTargetDatabaseName(req.getTargetDatabaseName().trim());
        }
        if (StringUtils.isNotEmpty(req.getTreeId())) {
            //判断是否为整数 是整数返回true,否则返回false
            Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
            if (pattern.matcher(req.getTreeId().trim()).matches()) {
                req.setTreeId(req.getTreeId()); //id
            } else {
                req.setTreeName(req.getTreeId().trim());//节点名称
                req.setTreeId("");
            }
        }

        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        List<RealTimeSyncListDto> list = dmpRealtimeSyncInfoDao.queryRealTimeSyncListPage(req);
        PageInfo<RealTimeSyncListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 根据多个实时任务id查询任务信息
     *
     * @return
     * @author Bellamy
     */
    @Override
    public List<DmpRealtimeSyncInfo> queryListById(String[] ids) throws Exception {
        return dmpRealtimeSyncInfoDao.queryListById(ids);
    }

    /**
     * 获取数据源和目标数据源下拉框
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @Override
    public JsonResult<List<DataSourceNameListDto>> queryDatasourceNameList(String projectId, String type) throws Exception {
        List<DataSourceNameListDto> list = dmpRealtimeSyncInfoDao.queryDatasourceNameList(projectId, type);
        return new JsonResult(list);
    }

    /**
     * 新增--选择源表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @Override
    public JsonResult<RealTimeSyncModel> queryTableInfoByParams(RealTimeTableInfoReq req) throws Exception {
        Map<String, Map<String, String>> paramsMap = null;
        //根据来源数据源id获取数据信息
        RealTimeSyncDataSourceModel sourceDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(req.getSrcDatasourceId());
        sourceDbInfo.setPassword(new BaseService().decode(sourceDbInfo.getPassword(), publicKey));

        Map map = new HashMap();
        map.put("srcDatasourceId", req.getSrcDatasourceId());
        map.put("targetDatasourceId", req.getTargetDatasourceId());
        //数据源对应的表详细信息  判断表的记录在同步任务中是否存在记录,如果已经存在,说明同步过
        List<Map> list = dmpRealtimeSyncInfoDao.queryRealTimeInfoByDataSourceId(map);
        if (list.size() > 0 && list != null) {
            paramsMap = new HashMap<>();
            for (Map item : list) {
                String srcTableName = item.get("srcTableName").toString();
                paramsMap.put(srcTableName, item);
            }
        }
        Map<String, String> blackMapSetting = null;
        //查询来源数据源的黑名单表
        Map blackTableMap = dmpRealtimeSyncInfoDao.queryBlackTableByDataSourceId(req.getSrcDatasourceId());
        if (blackTableMap.size() > 0 && blackTableMap != null) {
            String blacklistTable = blackTableMap.get("blacklistTable").toString();
            if (StringUtils.isNotEmpty(blacklistTable)) {
                String[] blacklistTableArr = blacklistTable.split(",");
                if (blacklistTableArr.length > 0 && blacklistTableArr != null) {
                    blackMapSetting = new HashMap<>();
                    for (String str : blacklistTableArr) {
                        blackMapSetting.put(str, str);
                    }
                }
            }
        }
        //来源数据源下的表信息
        List<RealTimeSyncTableModel> dataSourceTables = DBUtil.getDataSourceTables(sourceDbInfo, req.getToQueryTableName(), true, paramsMap, req.getBlacklistTables(), req.getSelectTablesName(), blackMapSetting);

        RealTimeSyncModel realTimeSyncModel = new RealTimeSyncModel();
        realTimeSyncModel.setSrcdatasourceId(Integer.valueOf(req.getSrcDatasourceId()));
        realTimeSyncModel.setDataSourceName(sourceDbInfo.getDatasourceName());
        realTimeSyncModel.setDbName(sourceDbInfo.getDbName());
        realTimeSyncModel.setTables(dataSourceTables); //来源数据源下的表信息
        return new JsonResult(realTimeSyncModel);
    }

    /**
     * 新增--选择目标信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @Override
    public JsonResult selectTargetDatasourceInfo(String projectId) throws Exception {
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(projectId));
        String kafkaConnectUrl = dmpProjectSystemInfo.getKafkaConnectorUrl(); //kafka 连接信息
        String[] arr = kafkaConnectUrl.split(",");
        return new JsonResult(arr);
    }

    /**
     * 设置成黑名单时 ,检查是否有同步任务存在运行
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @Override
    public JsonResult queryRealTimeInfoByDataSourceId(String srcDatasourceId, String sourceTableName) throws Exception {
        Map map = new HashMap();
        map.put("srcDatasourceId", srcDatasourceId); //来源表 数据源id
        map.put("sourceTableName", sourceTableName); // 来源表
        List<Map> list = dmpRealtimeSyncInfoDao.queryRealTimeInfoByDataSourceId(map);
        boolean flag = false;
        if (list.size() > 0 && list != null) {
            flag = true;
        }
        return new JsonResult(flag);
    }

    /**
     * 保存实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult addRealTimeTask(Map<String, Object> params) throws Exception {
        String srcDataSourceId = params.get("srcDataSourceId").toString(); // 来源id
        String targetDataSourceId = params.get("targetDataSourceId").toString(); // 目标源id
        Long projectId = Long.valueOf(params.get("projectId").toString()); //项目id
        String connectorUrl = (String) params.get("connectorUrl"); //connectorUrl连接信息

        //根据来源数据源id获取数据信息
        RealTimeSyncDataSourceModel sourceDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(srcDataSourceId);
        if (sourceDbInfo == null) {
            throw new RuntimeException("来源数据设置信息不存在！");
        }
        sourceDbInfo.setPassword(new BaseService().decode(sourceDbInfo.getPassword(), publicKey));

        //根据目标数据源id获取数据信息
        RealTimeSyncDataSourceModel targetDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(targetDataSourceId);
        if (targetDbInfo == null) {
            throw new RuntimeException("目标数据源设置信息不存在！");
        }
        targetDbInfo.setPassword(new BaseService().decode(targetDbInfo.getPassword(), publicKey));

        //根据projectId查询项目系统配置信息
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(projectId);
        params.put("connectorSecurityFlag", dmpProjectSystemInfo.getKerberosIsenable()); //安全验证开关,是否启用KERBEROS

       /* //保存实时同步任务处理数
        DmpRealtimeSyncHandleCount countModel = new DmpRealtimeSyncHandleCount();
        countModel.setUuid(UUID.randomUUID().toString());
        countModel.setToSubmit(1);
        countModel.setCreTime(new Date());
        //初始化当前任务记录数
        List<Map<String, String>> tables = (List<Map<String, String>>) params.get("tables");
        if (!CollectionUtils.isEmpty(tables)) {
            countModel.setToSubmit(countModel.getToSubmit() + tables.size());
        }
        dmpRealtimeSyncHandleCountDao.insert(countModel);
        logger.info("###################添加实时同步任务处理数结束{}" + countModel.toString() + "############");*/

        //connect1@http://172.18.104.130:9993/connectors
        if (StringUtils.isNotEmpty(connectorUrl)) {
            if (connectorUrl.contains("@")) {
                connectorUrl = connectorUrl.split("@")[1];
            }
        }
        //提交源到源的connector
        Long realtimeId = submitDatasource2DatasourceToConnector(projectId, sourceDbInfo, targetDbInfo, dmpProjectSystemInfo, connectorUrl, params);
        if (realtimeId != null) {
            //处理已选择的表信息
            submitNoSelectTable(realtimeId, projectId, sourceDbInfo, targetDbInfo, dmpProjectSystemInfo, connectorUrl, params);
        }
        return new JsonResult();
    }

    private Long submitDatasource2DatasourceToConnector(Long projectId, RealTimeSyncDataSourceModel sourceDbInfo, RealTimeSyncDataSourceModel targetDbInfo, DmpProjectSystemInfo dmpProjectSystemInfo, String connectorUrl, Map<String, Object> params) throws Exception {
        Integer srcDataSourceId = sourceDbInfo.getId();
        Integer targetDataSourceId = targetDbInfo.getId();

       /* //保存实时同步任务提交结果
        DmpRealtimeSyncSubmitResult dmpRealtimeSyncSubmitResult = new DmpRealtimeSyncSubmitResult();
        dmpRealtimeSyncSubmitResult.setUuid(uuid);
        dmpRealtimeSyncSubmitResult.setStatus(2);
        dmpRealtimeSyncSubmitResult.setType(1);
        dmpRealtimeSyncSubmitResult.setCreatedTime(new Date());
        dmpRealtimeSyncSubmitResult.setSubmitResult(sourceDbInfo.getDbName() + "----->" + sourceDbInfo.getDbName() + " 提交中");
        dmpRealtimeSyncHandleCountDao.insertRealtimeSyncSubmitResult(dmpRealtimeSyncSubmitResult);
        logger.info("###################添加实时同步任务提交结果 结束{}" + dmpRealtimeSyncSubmitResult.toString() + "############");

        DmpRealtimeSyncHandleCount handleCount = dmpRealtimeSyncHandleCountDao.queryById(uuid);
        if (handleCount != null) {
            handleCount.setToSubmit(handleCount.getToSubmit() - 1);
            handleCount.setSubmiting(handleCount.getSubmiting() + 1);
            dmpRealtimeSyncHandleCountDao.update(handleCount);
            logger.info("###################修改实时同步任务处理数--结束{}" + handleCount.toString() + "############");
        }*/

        logger.info("###################开始--同步数据源到数据源任务################### ");
        Long realtiemId = null; //同步任务id
        //源数据源到数据源同步connector信息
        //解析黑名单表
        Map blacklistTablesInfo = getBlackListTableInfo(sourceDbInfo, params);

        //解析已选择表
        Map selectlistTablesInfo = getSelectListTableInfo(sourceDbInfo, params);

        Map<String, String> dataModelMap = new HashMap<>();
        dataModelMap.put("kafkaBootstrapServers", dmpProjectSystemInfo.getKafkaBootstrapServers());//kafka 地址
        dataModelMap.put("registerUrl", dmpProjectSystemInfo.getKafkaSchemaRegisterUrl()); //kafka connector schema 注册地址
        dataModelMap.put("blacklistTables", blacklistTablesInfo.get("connectorBlacklistTables").toString()); //设置的黑名单表
        dataModelMap.put("blacklistTableCount", blacklistTablesInfo.get("blacklistTableCount").toString()); //黑名单表数量
        //设置的白名单表   在模板里进行判比较黑名单表和白名单表的数量，谁小就用谁
        dataModelMap.put("whitelistTablesConut", selectlistTablesInfo.get("whitelistTablesConut").toString()); //已选择表数量
        dataModelMap.put("connectorWhitelistTables", selectlistTablesInfo.get("connectorWhitelistTables").toString()); //设置的已选择表
        //选择的来源数据信息
        dataModelMap.put("dbHost", sourceDbInfo.getHost());
        dataModelMap.put("dbPort", sourceDbInfo.getPort());
        dataModelMap.put("dbUserName", sourceDbInfo.getUserName());
        dataModelMap.put("dbPassWord", sourceDbInfo.getPassword());
        dataModelMap.put("dataSourceId", sourceDbInfo.getId().toString());
        dataModelMap.put("datasourceName", sourceDbInfo.getDatasourceName());
        dataModelMap.put("dbName", sourceDbInfo.getDbName());
        dataModelMap.put("connectorSecurityFlag", (String) params.get("connectorSecurityFlag")); //安全验证开关,是否启用KERBEROS

        //前端定义的sourceConnectorName前缀
        String sourceName = (String) params.get("sourceName");
        dataModelMap.put("sourceName", sourceName);
        //source kafak topic
        String topic = sourceDbInfo.getDatasourceName() + "_" + sourceName + "." + sourceDbInfo.getDbName() + ".databasehistory";
        dataModelMap.put("topic", topic);
        String sourceConnectorName = "debezium-connector-" + sourceDbInfo.getDatasourceName() + "_" + sourceName + "-" + sourceDbInfo.getDbName();

        //查询同步任务信息表是否存在类型是数据源到数据源，srcDataSourceId，targetDataSourceId一样的信息  如果  存在就不发起请求
        Map<String, Object> queryParam = new HashMap<>();
        queryParam.put("srcDataSourceId", srcDataSourceId);
        queryParam.put("sourceConnectorName", sourceConnectorName);
        Map dataMap = dmpRealtimeSyncInfoDao.selecltRealtimeSyncInfoByParams(queryParam);
        //不存在同一个数据源的同步任务 没查到信息
        if (dataMap == null) {
            dataModelMap.put("name", sourceDbInfo.getDatasourceName() + "_" + sourceName + "-" + sourceDbInfo.getDbName()); //source connector name
            String dataSource2DataSourceJsonStr = freemakerJson("source", dataModelMap); //使用freemaker模板生成 kafka connector 请求参数

            Map<String, Object> dataSource2DataSourceResult = RestClient.post(connectorUrl, dataSource2DataSourceJsonStr);
            String connectorJobId = getConnectorJobId(dataSource2DataSourceResult);
            //请求接口正常则保存数据，否则失败
            if (StringUtils.isNotEmpty(connectorJobId)) {
                DmpRealtimeSyncInfo saveBody = new DmpRealtimeSyncInfo();
                saveBody.setSrcDatasourceId(srcDataSourceId);
                saveBody.setSrcDatasourceName(sourceDbInfo.getDatasourceName());
                saveBody.setSrcDatabaseType(sourceDbInfo.getDataSourceTypeName());
                saveBody.setSrcDatabaseName(sourceDbInfo.getDbName());
                saveBody.setTargetDatasourceId(targetDataSourceId);
                saveBody.setTargetDatasourceName(targetDbInfo.getDatasourceName());
                saveBody.setTargetDatabaseType(targetDbInfo.getDataSourceTypeName());
                saveBody.setTargetDatabaseName(targetDbInfo.getDbName());
                saveBody.setType(1);
                saveBody.setStatus("SUBMIT_SUCCESS");
                saveBody.setConnectorJsonData(dataSource2DataSourceJsonStr);
                saveBody.setProjectId(projectId);
                saveBody.setConnectorUrl(connectorUrl);
                saveBody.setSrcTopicName(topic);
                saveBody.setSourceTypeName(sourceDbInfo.getDataSourceTypeName());
                saveBody.setTargetTypeName("Kafka");
                saveBody.setConnectorJobId(connectorJobId);
                saveBody.setConnectorJsonData(dataSource2DataSourceJsonStr);
                saveBody.setCreateTime(new Date());
                saveBody.setCrePerson(SessionUtils.getCurrentUserId());
                dmpRealtimeSyncInfoDao.insert(saveBody);
                realtiemId = Long.valueOf(saveBody.getId());
                logger.info("###################保存实时同步任务--结束 ################");

                Map blacklist = new HashMap();
                blacklist.put("creTime", new Date());
                blacklist.put("realtimeId", saveBody.getId());
                blacklist.put("crePerson", SessionUtils.getCurrentUserId());
                blacklist.put("datasourceId", srcDataSourceId);
                blacklist.put("blacklistTable", blacklistTablesInfo.get("blacklistTables").toString());
                dmpRealtimeSyncInfoDao.insertRealtimeBlackList(blacklist);
                logger.info("###################保存实时同步黑名单数据--结束 ################");
            } else {
                throw new RuntimeException("提交失败！");
            }
        } else {
            throw new RuntimeException("存在相同的实时任务数据！");
        }
        return realtiemId;
    }

    /**
     * 解析黑名单表
     *
     * @return
     * @author Bellamy
     * @since 2021-01-11
     */
    public Map getBlackListTableInfo(RealTimeSyncDataSourceModel sourceDbInfo, Map<String, Object> params) {
        Map returnMap = new HashMap();
        String blacklistTables = "";
        String connectorBlacklistTables = "";
        int blacklistTableCount = 0;
        if (params.containsKey("blacklistTables")) {
            blacklistTables = (String) params.get("blacklistTables");
            if (StringUtils.isNotEmpty(blacklistTables)) {
                StringBuffer sb = new StringBuffer();
                String[] blacklistTableArr = blacklistTables.split(",");
                blacklistTableCount = blacklistTableArr.length;
                if (blacklistTableArr != null && blacklistTableCount >= 1) {
                    for (String string : blacklistTableArr) {
                        sb.append(sourceDbInfo.getDbName());
                        sb.append(".");
                        sb.append(string);
                        sb.append(",");
                    }
                }
                if (sb.toString().contains(",")) {
                    connectorBlacklistTables = sb.toString().substring(0, sb.toString().length() - 1);
                }
            }
        }
        returnMap.put("blacklistTables", blacklistTables);
        returnMap.put("connectorBlacklistTables", connectorBlacklistTables);
        returnMap.put("blacklistTableCount", blacklistTableCount);
        logger.info("###################解析黑名单表--结束{}" + connectorBlacklistTables + " ################");
        return returnMap;
    }

    /**
     * 解析已选择表
     *
     * @return
     * @author Bellamy
     * @since 2021-01-11
     */
    public Map getSelectListTableInfo(RealTimeSyncDataSourceModel sourceDbInfo, Map<String, Object> params) {
        Map returnMap = new HashMap();

        String connectorWhitelistTables = "";
        int whitelistTablesConut = 0;
        if (params.containsKey("tables")) {
            List<Map<String, String>> tables = (List<Map<String, String>>) params.get("tables");
            if (!CollectionUtils.isEmpty(tables)) {
                whitelistTablesConut = tables.size();
                StringBuffer sb = new StringBuffer();
                for (Map<String, String> temp : tables) {
                    sb.append(sourceDbInfo.getDbName());
                    sb.append(".");
                    sb.append(temp.get("sourceTableName"));
                    sb.append(",");
                }
                if (sb.toString().contains(",")) {
                    connectorWhitelistTables = sb.toString().substring(0, sb.toString().length() - 1);
                }
            }
        }
        returnMap.put("connectorWhitelistTables", connectorWhitelistTables);
        returnMap.put("whitelistTablesConut", whitelistTablesConut);
        logger.info("###################解析已选择表--结束{} " + connectorWhitelistTables + "################");
        return returnMap;
    }

    /**
     * 使用freemaker模板生成 kafka connector 请求参数
     *
     * @param type      模板类型
     * @param dataModel 模板里定义的变量数据对象
     * @return
     * @author Bellamy
     */
    public String freemakerJson(String type, Map<String, String> dataModel) {
        StringWriter stringWriter = new StringWriter();
        try {

            Template template = freeMarkerConfig.getConfiguration().getTemplate(type + "_connector.ftl");
            if (template != null) {
                try {
                    template.process(dataModel, stringWriter);
                } catch (TemplateException e) {
                    e.printStackTrace();
                }
            }
        } catch (TemplateNotFoundException e) {
            e.printStackTrace();
        } catch (MalformedTemplateNameException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return stringWriter.toString();
    }

    /**
     * 根据返回结果  获取返回的connectorName
     *
     * @param resultMap
     * @return
     * @author Bellamy
     */
    public String getConnectorJobId(Map<String, Object> resultMap) {
        if (CollectionUtils.isEmpty(resultMap)) {
            return null;
        }
        if (!resultMap.containsKey("name")) {
            return null;
        }
        return (String) resultMap.get("name");
    }

    /**
     * 处理已选择的表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    private JsonResult submitNoSelectTable(Long realtimeId, Long projectId, RealTimeSyncDataSourceModel sourceDbInfo, RealTimeSyncDataSourceModel targetDbInfo, DmpProjectSystemInfo dmpProjectSystemInfo, String connectorUrl, Map<String, Object> params) throws Exception {
        logger.info("###################处理已选择的表信息################");

       /* //源数据源到数据源同步connector信息  ----开始------
        Map<String, String> tableToTableConfigArg = new HashMap<>();
        //默认配置参数  --{kudu master 的地址} 10.0.108.61,10.0.108.62
        String endpoint = targetDbInfo.getEndpoint();
        //--{share key} CRNET_SECRET_KEY
        String shareKey = (String) getDictionaryParamValueByKey("CRNET_SECRET_KEY");
        tableToTableConfigArg.put("shareKey", shareKey);
        //--{impala 地址} 10.0.108.62:21050
        //目标数据源类型
        String storeType = targetDbInfo.getDataSourceTypeName().toLowerCase();
        String jdbcUrl = targetDbInfo.getJdbcUrl();
        if ("kudu".equals(storeType)) {
            tableToTableConfigArg.put("kuduMaster", endpoint);
            String impalaUrl = jdbcUrl;
            if (!org.springframework.util.StringUtils.isEmpty(impalaUrl)) {
                impalaUrl = impalaUrl.replace("jdbc:impala://", "");
                if (impalaUrl.contains("/")) {
                    impalaUrl = impalaUrl.substring(0, impalaUrl.indexOf("/"));
                }
                tableToTableConfigArg.put("impalaUrl", impalaUrl);
            }
            tableToTableConfigArg.put("impalaFqdn", targetDbInfo.getAccessId());
            //--{schem registry url} http://10.0.108.61:9881
            tableToTableConfigArg.put("registryUrl", dmpProjectSystemInfo.getKafkaSchemaRegisterUrl());
        } else if ("hdfs".equals(storeType)) {
            tableToTableConfigArg.put("jdbcUrl", jdbcUrl);
            tableToTableConfigArg.put("databaseName", targetDbInfo.getDbName());
            tableToTableConfigArg.put("hiveMetastoreUris", endpoint);
            tableToTableConfigArg.put("hdfsNamenodePrincipal", targetDbInfo.getAccessId());
            tableToTableConfigArg.put("keytabLocation", targetDbInfo.getAccessKey());
        }
        tableToTableConfigArg.put("connectorSecurityFlag", (String) params.get("connectorSecurityFlag"));*/

        //已选择的表信息
        List<Map<String, String>> tables = (List<Map<String, String>>) params.get("tables");
        if (tables.size() > 0 && tables != null) {
            for (Map<String, String> str : tables) {
                DmpRealtimeSyncSelectTable selectTable = new DmpRealtimeSyncSelectTable();
                selectTable.setCrePerson(SessionUtils.getCurrentUserId());
                selectTable.setCreTime(new Date());
                selectTable.setSrcTableName(str.get("sourceTableName"));
                selectTable.setTargetTableName(str.get("targetTableName"));
                selectTable.setRealtimeId(realtimeId);
                selectTable.setDesensitizationField(str.get("desensitizationField"));
                selectTable.setPkName(str.get("pkName"));
                selectTable.setArithmetic(str.get("arithmetic"));
                dmpRealtimeSyncInfoDao.insertRealtimeSelectTable(selectTable);
            }
        }
        logger.info("###################处理已选择的表信息--结束################");
        return new JsonResult();
    }

    /**
     * 编辑实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult updateRealTimeTask(Map<String, Object> params) throws Exception {
        String srcDataSourceId = params.get("srcDataSourceId").toString(); //来源id
        String targetDataSourceId = params.get("targetDataSourceId").toString(); //目标源id
        Long projectId = Long.valueOf(params.get("projectId").toString()); //项目id
        String connectorUrl = (String) params.get("connectorUrl"); //connectorUrl连接信息

        //根据来源数据源id获取数据信息
        RealTimeSyncDataSourceModel sourceDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(srcDataSourceId);
        if (sourceDbInfo == null) {
            throw new RuntimeException("来源数据设置信息不存在！");
        }
        sourceDbInfo.setPassword(new BaseService().decode(sourceDbInfo.getPassword(), publicKey));

        //根据目标数据源id获取数据信息
        RealTimeSyncDataSourceModel targetDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(targetDataSourceId);
        if (targetDbInfo == null) {
            throw new RuntimeException("目标数据源设置信息不存在！");
        }
        targetDbInfo.setPassword(new BaseService().decode(targetDbInfo.getPassword(), publicKey));

        //根据projectId查询项目系统配置信息
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(projectId);
        params.put("connectorSecurityFlag", dmpProjectSystemInfo.getKerberosIsenable()); //安全验证开关,是否启用KERBEROS

        //connect1@http://172.18.104.130:9993/connectors
        if (StringUtils.isNotEmpty(connectorUrl)) {
            if (connectorUrl.contains("@")) {
                connectorUrl = connectorUrl.split("@")[1];
            }
        }
        //提交源到源的connector
        JsonResult realtimeId = updateDatasource2DatasourceToConnector(projectId, sourceDbInfo, targetDbInfo, dmpProjectSystemInfo, connectorUrl, params);

        //编辑 已选择表信息
        updateNoSelectTable(params);
        return new JsonResult();
    }

    private JsonResult updateDatasource2DatasourceToConnector(Long projectId, RealTimeSyncDataSourceModel sourceDbInfo, RealTimeSyncDataSourceModel targetDbInfo, DmpProjectSystemInfo dmpProjectSystemInfo, String connectorUrl, Map<String, Object> params) throws Exception {
        Integer srcDataSourceId = sourceDbInfo.getId(); //来源数据源id
        Integer targetDataSourceId = targetDbInfo.getId(); //目标数据源id

        String realtiemId = params.get("taskId").toString(); //同步任务id
        //解析黑名单表
        Map blacklistTablesInfo = getBlackListTableInfo(sourceDbInfo, params);

        //解析已选择表
        Map selectlistTablesInfo = getSelectListTableInfo(sourceDbInfo, params);

        Map<String, String> dataModelMap = new HashMap<>();
        dataModelMap.put("kafkaBootstrapServers", dmpProjectSystemInfo.getKafkaBootstrapServers());//kafka 地址
        dataModelMap.put("registerUrl", dmpProjectSystemInfo.getKafkaSchemaRegisterUrl()); //kafka connector schema 注册地址
        dataModelMap.put("blacklistTables", blacklistTablesInfo.get("connectorBlacklistTables").toString()); //设置的黑名单表
        dataModelMap.put("blacklistTableCount", blacklistTablesInfo.get("blacklistTableCount").toString()); //黑名单表数量
        //设置的白名单表   在模板里进行判比较黑名单表和白名单表的数量，谁小就用谁
        dataModelMap.put("whitelistTablesConut", selectlistTablesInfo.get("whitelistTablesConut").toString()); //已选择表数量
        dataModelMap.put("connectorWhitelistTables", selectlistTablesInfo.get("connectorWhitelistTables").toString()); //设置的已选择表
        //选择的来源数据信息
        dataModelMap.put("dbHost", sourceDbInfo.getHost()); //IP
        dataModelMap.put("dbPort", sourceDbInfo.getPort()); //端口
        dataModelMap.put("dbUserName", sourceDbInfo.getUserName());
        dataModelMap.put("dbPassWord", sourceDbInfo.getPassword());
        dataModelMap.put("dataSourceId", sourceDbInfo.getId().toString());
        dataModelMap.put("datasourceName", sourceDbInfo.getDatasourceName()); //数据源名称
        dataModelMap.put("dbName", sourceDbInfo.getDbName()); //数据库名称
        dataModelMap.put("connectorSecurityFlag", (String) params.get("connectorSecurityFlag")); //安全验证开关,是否启用KERBEROS
        //前端定义的sourceConnectorName前缀
        String sourceName = (String) params.get("sourceName");
        dataModelMap.put("sourceName", sourceName);
        //source kafak topic
        String topic = sourceDbInfo.getDatasourceName() + "_" + sourceName + "." + sourceDbInfo.getDbName() + ".databasehistory";
        dataModelMap.put("topic", topic);
        dataModelMap.put("name", sourceDbInfo.getDatasourceName() + "_" + sourceName + "-" + sourceDbInfo.getDbName()); //source connector name
        String dataSource2DataSourceJsonStr = freemakerJson("source", dataModelMap); //使用freemaker模板生成 kafka connector 请求参数

        Map<String, Object> dataSource2DataSourceResult = RestClient.post(connectorUrl, dataSource2DataSourceJsonStr);
        String connectorJobId = getConnectorJobId(dataSource2DataSourceResult);
        //请求接口正常则保存数据，否则失败
        if (StringUtils.isNotEmpty(connectorJobId)) {
            DmpRealtimeSyncInfo saveBody = new DmpRealtimeSyncInfo();
            saveBody.setSrcDatasourceId(srcDataSourceId);
            saveBody.setSrcDatasourceName(sourceDbInfo.getDatasourceName());
            saveBody.setSrcDatabaseType(sourceDbInfo.getDataSourceTypeName());
            saveBody.setSrcDatabaseName(sourceDbInfo.getDbName());
            saveBody.setTargetDatasourceId(targetDataSourceId);
            saveBody.setTargetDatasourceName(targetDbInfo.getDatasourceName());
            saveBody.setTargetDatabaseType(targetDbInfo.getDataSourceTypeName());
            saveBody.setTargetDatabaseName(targetDbInfo.getDbName());
            saveBody.setStatus("SUBMIT_SUCCESS");
            saveBody.setConnectorJsonData(dataSource2DataSourceJsonStr);
            saveBody.setProjectId(projectId);
            saveBody.setConnectorUrl(connectorUrl);
            saveBody.setSrcTopicName(topic);
            saveBody.setSourceTypeName(sourceDbInfo.getDataSourceTypeName());
            saveBody.setTargetTypeName("Kafka");
            saveBody.setConnectorJobId(connectorJobId);
            saveBody.setConnectorJsonData(dataSource2DataSourceJsonStr);
            saveBody.setUpdateTime(new Date());
            saveBody.setId(Integer.valueOf(realtiemId));
            saveBody.setUptPerson(SessionUtils.getCurrentUserId());
            dmpRealtimeSyncInfoDao.update(saveBody);
            logger.info("###################修改实时同步任务--结束 ################");

            Map blacklist = new HashMap();
            blacklist.put("uptTime", new Date());
            blacklist.put("uptPerson", SessionUtils.getCurrentUserId());
            blacklist.put("realtimeId", realtiemId);
            blacklist.put("datasourceId", srcDataSourceId);
            blacklist.put("blacklistTable", blacklistTablesInfo.get("blacklistTables").toString());
            dmpRealtimeSyncInfoDao.updateRealtimeBlackList(blacklist);
            logger.info("###################修改实时同步黑名单数据--结束 ################");
        } else {
            throw new RuntimeException("提交失败！");
        }
        return new JsonResult();
    }

    /**
     * 修改处理已选择的表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-11
     */
    private JsonResult updateNoSelectTable(Map<String, Object> params) throws Exception {
        logger.info("###################处理已选择的表信息################");
        String realtimeId = params.get("taskId").toString(); //同步任务id
        //已选择的表信息
        List<Map<String, String>> tables = (List<Map<String, String>>) params.get("tables");
        if (tables.size() > 0 && tables != null) {
            for (Map<String, String> str : tables) {
                DmpRealtimeSyncSelectTable selectTable = new DmpRealtimeSyncSelectTable();
                selectTable.setUptPerson(SessionUtils.getCurrentUserId());
                selectTable.setUptTime(new Date());
                selectTable.setSrcTableName(str.get("sourceTableName"));
                selectTable.setTargetTableName(str.get("targetTableName"));
                selectTable.setRealtimeId(Long.valueOf(realtimeId));
                selectTable.setDesensitizationField(str.get("desensitizationField"));
                selectTable.setPkName(str.get("pkName"));
                selectTable.setArithmetic(str.get("arithmetic"));
                dmpRealtimeSyncInfoDao.updateRealtimeSelectTable(selectTable);
            }
        }
        logger.info("###################处理已选择的表信息--结束################");
        return new JsonResult();
    }

    /**
     * 编辑--数据回显,通过id查询任务相关信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-12
     */
    @Override
    public JsonResult<RealTimeEditDataEchoDto> selectRealtimeTaskById(String taskId) throws Exception {
        RealTimeEditDataEchoDto returnModel = new RealTimeEditDataEchoDto();
        List<Map> list = dmpRealtimeSyncInfoDao.selectRealtimeTaskById(taskId);
        if (list.size() > 0 && list != null) {
            Map realtimeMap = list.get(0);
            returnModel.setBlacklistTable(realtimeMap.get("blacklistTable").toString());
            returnModel.setSrcDatasourceId(realtimeMap.get("srcDatasourceId").toString());
            returnModel.setSrcDatasourceName(realtimeMap.get("srcDatasourceName").toString());
            returnModel.setTargetDatasourceId(realtimeMap.get("targetDatasourceId").toString());
            returnModel.setTargetDatasourceName(realtimeMap.get("targetDatasourceName").toString());
            returnModel.setId(realtimeMap.get("id").toString());
            List<DmpRealtimeSyncSelectTable> selectList = new ArrayList<>();
            for (int i = 0; i < list.size(); i++) {
                Map str = list.get(i);
                DmpRealtimeSyncSelectTable selectTable = new DmpRealtimeSyncSelectTable();
                selectTable.setDesensitizationField(str.get("desensitizationField").toString());
                selectTable.setArithmetic(str.get("arithmetic").toString());
                selectTable.setPkName(str.get("pkName").toString());
                selectTable.setSrcTableName(str.get("srcTableName").toString());
                selectTable.setTargetTableName(str.get("targetTableName").toString());
                selectList.add(selectTable);
                returnModel.setSelectTable(selectList);
            }

        }
        return new JsonResult(returnModel);
    }

    /**
     * 批量删除数据源
     *
     * @return
     * @author Bellamy
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public boolean deleteByrealTaskId(String realTaskId) throws Exception {
        Map map = new HashMap();
        String[] ids = realTaskId.split(",");
        map.put("ids", ids);
        map.put("dataStatus", DelFlagEnum.YES.getValue());
        return dmpRealtimeSyncInfoDao.deleteByrealTaskId(map) > 0;
    }
}