package com.jz.dmp.modules.service.sys.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.ResultCode;
import com.jz.common.constant.StatuConstant;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.web.SessionUtils;
import com.jz.dmp.modules.controller.sys.bean.DmpMemberBatch;
import com.jz.dmp.modules.controller.sys.bean.DmpMemberDto;
import com.jz.dmp.modules.controller.sys.bean.DmpMemberRequest;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.dao.sys.DmpMemberMapper;
import com.jz.dmp.modules.dao.sys.DmpRoleMapper;
import com.jz.dmp.modules.model.DmpMember;
import com.jz.dmp.modules.model.DmpPermission;
import com.jz.dmp.modules.model.DmpProject;
import com.jz.dmp.modules.model.DmpRole;
import com.jz.dmp.modules.service.sys.DmpMemberService;
import com.jz.dmp.modules.service.sys.DmpPermissionService;
import com.jz.dmp.modules.service.sys.DmpRoleService;

/**
 * 用户表服务的实现?
 * @author ybz
 *
 */
@Service("dmpMemberService")
public class DmpMemberServiceImpl extends BaseService implements DmpMemberService {
	
	private static Logger logger = LoggerFactory.getLogger(DmpMemberServiceImpl.class);
	
	@Autowired
	private DmpMemberMapper dmpMemberMapper;
	
	@Autowired
	private DmpRoleMapper dmpRoleMapper;
	
	@Autowired
	private DmpProjectDao dmpProjectDao;
	
	@Autowired
	private DmpRoleService dmpRoleService;
	
	@Autowired
	private DmpPermissionService dmpPermissionService;

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberService#findList(com.ycxc.vmts.controller.bean.DmpMemberRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<Map<String, Object>> findListWithPage(DmpMemberRequest dmpMemberRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<Map<String, Object>> baseBeanResponse = new BaseBeanResponse<>();
		
		Map<String, Object> param = new HashMap<>();
		//用户ID
		if (dmpMemberRequest.getUserId() != null) {
			param.put("userId", dmpMemberRequest.getUserId());
		}
		//用户名称
		if (!StringUtils.isEmpty(dmpMemberRequest.getUserName())) {
			param.put("userName", dmpMemberRequest.getUserName());
		}
		//登录密码
		if (!StringUtils.isEmpty(dmpMemberRequest.getUserPassword())) {
			param.put("userPassword", dmpMemberRequest.getUserPassword());
		}
		//真实姓名
		if (!StringUtils.isEmpty(dmpMemberRequest.getRealName())) {
			param.put("realName", dmpMemberRequest.getRealName());
		}
		//联系电话
		if (!StringUtils.isEmpty(dmpMemberRequest.getUserPhone())) {
			param.put("userPhone", dmpMemberRequest.getUserPhone());
		}
		//登录IP
		if (!StringUtils.isEmpty(dmpMemberRequest.getLoginIp())) {
			param.put("loginIp", dmpMemberRequest.getLoginIp());
		}
		//最近登录时间起
		if (dmpMemberRequest.getLoginTimeStart() != null) {
			param.put("loginTimeStart", dmpMemberRequest.getLoginTimeStart());
		}
		//最近登录时间止
		if (dmpMemberRequest.getLoginTimeEnd() != null) {
			param.put("loginTimeEnd", dmpMemberRequest.getLoginTimeEnd());
		}
		//是否登录锁定(0：否，1：是)
		if (!StringUtils.isEmpty(dmpMemberRequest.getLoginLocked())) {
			param.put("loginLocked", dmpMemberRequest.getLoginLocked());
		}
		//连续登录错误次数
		if (dmpMemberRequest.getLoginErrtimes() != null) {
			param.put("loginErrtimes", dmpMemberRequest.getLoginErrtimes());
		}
		//备注
		if (!StringUtils.isEmpty(dmpMemberRequest.getRemark())) {
			param.put("remark", dmpMemberRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpMemberRequest.getDataStatus())) {
			param.put("dataStatus", dmpMemberRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpMemberRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpMemberRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpMemberRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpMemberRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpMemberRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpMemberRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpMemberRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpMemberRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpMemberRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpMemberRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpMemberRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpMemberRequest.getUpdateTimeEnd());
		}
		//用户项目角色编码
		if (!StringUtils.isEmpty(dmpMemberRequest.getMpRoleCode())) {
			param.put("mpRoleCode", dmpMemberRequest.getMpRoleCode());
		}
		// 项目主键
		if (dmpMemberRequest.getProjectId() != null) {
			param.put("projectId", dmpMemberRequest.getProjectId());
		}
		
		
		PageHelper.startPage(dmpMemberRequest.getPageNum(), dmpMemberRequest.getPageSize());
		List<DmpMemberDto> list = dmpMemberMapper.findList(param);
		PageInfo<DmpMemberDto> pageInfo = new PageInfo<>(list);
		
		//获取所有用户角色统计信息
		List<Map<String, Object>> roleStatList = dmpRoleMapper.projRoleStat();
		
		Map<String, Object> resultMap = new HashMap<String, Object>();
		resultMap.put("pageInfo", pageInfo);
		resultMap.put("roleStatList", roleStatList);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(resultMap);
		return baseBeanResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberService#findList(com.ycxc.vmts.controller.bean.DmpMemberRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpMemberDto> findList(DmpMemberRequest dmpMemberRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpMemberDto> baseBeanResponse = new BaseBeanResponse<>();
		
		Map<String, Object> param = new HashMap<>();
		//用户ID
		if (dmpMemberRequest.getUserId() != null) {
			param.put("userId", dmpMemberRequest.getUserId());
		}
		//用户名称
		if (!StringUtils.isEmpty(dmpMemberRequest.getUserName())) {
			param.put("userName", dmpMemberRequest.getUserName());
		}
		//登录密码
		if (!StringUtils.isEmpty(dmpMemberRequest.getUserPassword())) {
			param.put("userPassword", dmpMemberRequest.getUserPassword());
		}
		//真实姓名
		if (!StringUtils.isEmpty(dmpMemberRequest.getRealName())) {
			param.put("realName", dmpMemberRequest.getRealName());
		}
		//联系电话
		if (!StringUtils.isEmpty(dmpMemberRequest.getUserPhone())) {
			param.put("userPhone", dmpMemberRequest.getUserPhone());
		}
		//登录IP
		if (!StringUtils.isEmpty(dmpMemberRequest.getLoginIp())) {
			param.put("loginIp", dmpMemberRequest.getLoginIp());
		}
		//最近登录时间起
		if (dmpMemberRequest.getLoginTimeStart() != null) {
			param.put("loginTimeStart", dmpMemberRequest.getLoginTimeStart());
		}
		//最近登录时间止
		if (dmpMemberRequest.getLoginTimeEnd() != null) {
			param.put("loginTimeEnd", dmpMemberRequest.getLoginTimeEnd());
		}
		//是否登录锁定(0：否，1：是)
		if (!StringUtils.isEmpty(dmpMemberRequest.getLoginLocked())) {
			param.put("loginLocked", dmpMemberRequest.getLoginLocked());
		}
		//连续登录错误次数
		if (dmpMemberRequest.getLoginErrtimes() != null) {
			param.put("loginErrtimes", dmpMemberRequest.getLoginErrtimes());
		}
		//备注
		if (!StringUtils.isEmpty(dmpMemberRequest.getRemark())) {
			param.put("remark", dmpMemberRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpMemberRequest.getDataStatus())) {
			param.put("dataStatus", dmpMemberRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpMemberRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpMemberRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpMemberRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpMemberRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpMemberRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpMemberRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpMemberRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpMemberRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpMemberRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpMemberRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpMemberRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpMemberRequest.getUpdateTimeEnd());
		}
		// 项目主键（用户过滤该项目下的成员）
		if (dmpMemberRequest.getUnChooseProjId() != null) {
			param.put("unChooseProjId", dmpMemberRequest.getUnChooseProjId());
		}
		
		List<DmpMemberDto> list = dmpMemberMapper.findList(param);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(list);
		return baseBeanResponse;
	}
	

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberService#add(com.ycxc.vmts.entity.DmpMember, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpMember> add(DmpMember dmpMember, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpMember> baseBeanResponse = new BaseBeanResponse<>();
		
		//判断账户是否已经存在
		DmpMember dmpMemberDb = dmpMemberMapper.findByUserName(dmpMember.getUsername());
		if (dmpMemberDb!=null) {
			baseBeanResponse.setCode(StatuConstant.CODE_DATA_EXISTED);
			baseBeanResponse.setMessage("该用户名已经存在");
			return baseBeanResponse;
		}
		
		/*$*setUserIdCodeReplace*$*/
		//创建人
		dmpMember.setCreateUserId(getHttpRequestUserId(httpRequest));
		//创建时间
		dmpMember.setCreateTime(new Date());
		
		dmpMemberMapper.insertSelective(dmpMember);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dmpMember);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberService#edit(com.ycxc.vmts.entity.DmpMember, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpMember> edit(DmpMember dmpMember, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpMember> baseBeanResponse = new BaseBeanResponse<>();
		
		//修改人
		dmpMember.setUpdateUserId(getHttpRequestUserId(httpRequest));
		//修改时间
		dmpMember.setUpdateTime(new Date());
		
		dmpMemberMapper.updateByPrimaryKeySelective(dmpMember);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("修改成功");
		baseBeanResponse.setData(dmpMember);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberService#findById(com.ycxc.vmts.controller.bean.DmpMemberRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpMemberDto> findById(DmpMemberRequest dmpMemberRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpMemberDto> baseBeanResponse = new BaseBeanResponse<>();
		
		DmpMemberDto dmpMemberDto = dmpMemberMapper.findById(dmpMemberRequest.getUserId());
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(dmpMemberDto);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberService#deleteById(com.ycxc.vmts.controller.bean.DmpMemberRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteById(Integer userId, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dmpMemberMapper.deleteByPrimaryKey(userId);
		
		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");
		
		return baseResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberService#deleteById(com.ycxc.vmts.controller.bean.DmpMemberRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse softDeleteById(Integer userId, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dmpMemberMapper.softDeleteByPrimaryKey(userId);
		
		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");
		
		return baseResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberService#addBatch(com.ycxc.vmts.controller.bean.DmpMemberVo, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpMemberBatch> addBatch(DmpMemberBatch dmpMemberBatch, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpMemberBatch> baseBeanResponse = new BaseBeanResponse<>();
		
		List<DmpMember> dmpMembers = dmpMemberBatch.getDmpMembers();
		
		if (CollectionUtils.isEmpty(dmpMembers)) {
			baseBeanResponse.setCode(ResultCode.PARAMS_ERROR.val());
			baseBeanResponse.setMessage("数据不能为空！");
			return baseBeanResponse;
		}
		
		dmpMembers = dmpMembers.stream().map(dmpMember -> {
			/*$*setUserIdCodeReplace*$*/
			//创建人
			try {
				dmpMember.setCreateUserId(getHttpRequestUserId(httpRequest));
			} catch (Exception e) {
				logger.info(e.getMessage());
				e.printStackTrace();
			}
			//创建时间
			dmpMember.setCreateTime(new Date());
			
			return dmpMember;
		}).collect(Collectors.toList());
		
		dmpMemberMapper.insertBatch(dmpMembers);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("批量新增成功");
		baseBeanResponse.setData(dmpMemberBatch);
		
		return baseBeanResponse;
	}


	/**
	 *通过用户名获取用户信息
	 */
	@Override
	public DmpMember findByUserName(String userName) throws Exception {
		DmpMember dmpMember = dmpMemberMapper.findByUserName(userName);
		return dmpMember;
	}


	/**
	 *根据用户获取默认项目ID
	 */
	@Override
	public Integer getDefProjectByUserName(String username) {
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("ownerId", username);
		List<DmpProject> projects = dmpProjectDao.getExampleQuery(params);
		if (CollectionUtils.isEmpty(projects)) {
			logger.info("该用户没有项目");
			return null;
		}
		return projects.get(0).getId();
	}


	/**
	 *用户项目切换
	 */
	@Override
	public BaseBeanResponse<DmpMember> switchProj(Integer projectId, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpMember> baseBeanResponse = new BaseBeanResponse<>();
		
		Authentication authentication = SessionUtils.getAuthentication();
		Object principal = authentication.getPrincipal();
		if (principal==null) {
			baseBeanResponse.setCode(StatuConstant.CODE_NOT_SIGNED);
			baseBeanResponse.setCode(StatuConstant.MSG_NOT_SIGNED);
			return baseBeanResponse;
		}
		DmpMember member = (DmpMember)principal;
		
		//设置登录用户默认项目
		member.setCurrentProjectId(projectId);
		//设置当前默认项目用户所拥有的角色
		List<DmpRole> memberProjectRoles = dmpRoleService.findMemberProjRolesByUser(member);
		member.setMemberProjectRoleList(memberProjectRoles);
		//设置用户在当前默认项目的资源
		List<DmpPermission> memberProjPermissions = dmpPermissionService.findPermissionByRoles(memberProjectRoles);
		member.setMemberProjectPermissionList(memberProjPermissions);
		
		List<DmpPermission> permissions = new ArrayList<DmpPermission>();
		if (!CollectionUtils.isEmpty(memberProjPermissions)) {
			String jsonStr = JSONObject.toJSONString(memberProjPermissions);
			List<DmpPermission> list = JSONObject.parseArray(jsonStr, DmpPermission.class);
			permissions.addAll(list);
		}
		if (!CollectionUtils.isEmpty(member.getMemberPermissionList())) {
			String jsonStr = JSONObject.toJSONString(member.getMemberPermissionList());
			List<DmpPermission> list = JSONObject.parseArray(jsonStr, DmpPermission.class);
			permissions.addAll(list);
		}
		member.setPermissionList(permissions);
		
		//修改session上用户信息
		//生成当所有授权
		List<GrantedAuthority> grantedAuthorities = new ArrayList<GrantedAuthority>(authentication.getAuthorities());
		//更改授权
		
		//生成新的认证信息
		Authentication newAuthentication = new UsernamePasswordAuthenticationToken(member, authentication.getCredentials(), grantedAuthorities);
		//重置认证信息
		SessionUtils.setAuthentication(newAuthentication);
		
		baseBeanResponse.setCode(StatuConstant.CODE_SUCCESS);
		baseBeanResponse.setMessage(StatuConstant.MSG_SUCCESS);
		baseBeanResponse.setData(member);
		
		return baseBeanResponse;
	}
	

	/**
	 *批量删除
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteByIds(List<Integer> idList, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpMemberMapper.deleteByIds(idList);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/**
	 *批量软删除
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse softDeleteByIds(List<Integer> idList, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dmpMemberMapper.softDeleteByIds(idList);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

	
}
